@@ -27,11 +27,12 @@
 from io import BufferedIOBase, BytesIO
 import math
 import struct
-from typing import Callable, Optional, Tuple, Union
+from typing import Any, Dict, Callable, List, Optional, Tuple, Union
+from typing_validation import validate
 
-import cid # type: ignore
+from multiformats import multicodec, CID
 
-from .encoding import EncodableType
+from .encoding import EncodableType, _dag_cbor_code
 from .utils import CBORDecodingError, DAGCBORDecodingError
 
 DecodeCallback = Callable[[EncodableType, int], None]
@@ -45,7 +46,8 @@ DecodeCallback = Callable[[EncodableType, int], None]
 
 def decode(stream_or_bytes: Union[BufferedIOBase, bytes], *,
            allow_concat: bool = False,
-           callback: Optional["DecodeCallback"] = None) -> EncodableType:
+           callback: Optional["DecodeCallback"] = None,
+           require_multicodec: bool = False) -> EncodableType:
     """
         Decodes and returns a single data item from the given `stream_or_bytes`, with the DAG-CBOR codec.
 
@@ -92,11 +94,22 @@ def decode(stream_or_bytes: Union[BufferedIOBase, bytes], *,
             >>> dag_cbor.decode(bytes_remaining)
             [0, 1]
         ```
+
+        If the optional keyword argument `require_multicodec` is `True`, the data being decoded must includes the multicodec code for 'dag-cbor'
+        (see [`multicodec.unwrap`](https://github.com/hashberg-io/multiformats#multicodec)).
     """
+    validate(stream_or_bytes, Union[BufferedIOBase, bytes])
+    validate(allow_concat, bool)
+    validate(require_multicodec, bool)
+    # validate(callback, Optional[DecodeCallback]) # TODO: not yet supported by typing_validation
     if isinstance(stream_or_bytes, bytes):
         stream: BufferedIOBase = BytesIO(stream_or_bytes)
     else:
         stream = stream_or_bytes
+    if require_multicodec:
+        code, _, stream = multicodec.unwrap_raw(stream)
+        if code != _dag_cbor_code:
+            raise DAGCBORDecodingError(f"Required 'dag-cbor' multicodec code {hex(_dag_cbor_code)}, unwrapped code {hex(code)} instead.")
     data, _ = _decode_item(stream, callback=callback)
     if allow_concat:
         return data
@@ -204,8 +217,8 @@ def _decode_str(stream: BufferedIOBase, length: int) -> Tuple[str, int]:
     return (res.decode(encoding="utf-8", errors="strict"), length)
 
 def _decode_list(stream: BufferedIOBase, length: int, *,
-                 callback: Optional[DecodeCallback]) -> Tuple[list, int]:
-    l: list = []
+                 callback: Optional[DecodeCallback]) -> Tuple[List[Any], int]:
+    l: List[Any] = []
     for i in range(length):
         try:
             item, _ = _decode_item(stream, callback=callback)
@@ -215,8 +228,8 @@ def _decode_list(stream: BufferedIOBase, length: int, *,
     return (l, 0)
 
 def _decode_dict(stream: BufferedIOBase, length: int,
-                 callback: Optional[DecodeCallback]) -> Tuple[dict, int]:
-    d: dict = {}
+                 callback: Optional[DecodeCallback]) -> Tuple[Dict[str, Any], int]:
+    d: Dict[str, Any] = {}
     for i in range(length):
         try:
             k, _ = _decode_item(stream, callback=callback)
@@ -233,7 +246,7 @@ def _decode_dict(stream: BufferedIOBase, length: int,
         raise DAGCBORDecodingError(f"Found only {len(d)} unique keys out of {length} key-value pairs.")
     return (d, 0)
 
-def _decode_cid(stream: BufferedIOBase, arg: int) -> Tuple[cid.cid.BaseCID, int]:
+def _decode_cid(stream: BufferedIOBase, arg: int) -> Tuple[CID, int]:
     if arg != 42:
         raise DAGCBORDecodingError(f"Error while decoding major type 0x6: tag {arg} is not allowed.")
     try:
@@ -242,7 +255,7 @@ def _decode_cid(stream: BufferedIOBase, arg: int) -> Tuple[cid.cid.BaseCID, int]
         raise CBORDecodingError("Error while decoding CID bytes.") from e
     if not isinstance(cid_bytes, bytes):
         raise DAGCBORDecodingError(f"Expected CID bytes, found data of type {type(cid_bytes)} instead.")
-    return (cid.from_bytes(cid_bytes), num_bytes_read)
+    return (CID.decode(cid_bytes), num_bytes_read)
 
 def _decode_bool_none(stream: BufferedIOBase, arg: int) -> Tuple[Optional[bool], int]:
     if arg == 20:
