@@ -27,31 +27,46 @@
 from io import BufferedIOBase, BytesIO
 import math
 import struct
-from typing import Optional, overload, Union
+from typing import Any, Dict, List, Optional, overload, Union
+from typing_validation import validate
 
-import cid # type: ignore
+from multiformats import varint, multicodec, CID
 
 from .utils import CBOREncodingError, DAGCBOREncodingError, _check_key_compliance
 
-EncodableType = Union[None, bool, int, float, bytes, str, list, dict, cid.cid.BaseCID]
+FlatEncodableType = Union[None, bool, int, float, bytes, str, CID]
+"""
+    Union of non-container Python types that can be encoded by this implementation of the DAG-CBOR codec:
+
+    ```py
+    typing.Union[NoneType, bool, int, float, bytes, str, multiformats.cid.CID]
+    ```
+"""
+
+EncodableType = Union[FlatEncodableType, List[Any], Dict[str, Any]]
 """
     Union of Python types that can be encoded by this implementation of the DAG-CBOR codec:
 
     ```py
-        EncodableType = Union[None, bool, int, float, bytes,
-                              str, list, dict, cid.cid.BaseCID]
+    typing.Union[NoneType, bool, int, float, bytes, str, multiformats.cid.CID,
+                 typing.List[typing.Any], typing.Dict[str, typing.Any]]
     ```
 """
 
+_dag_cbor_multicodec = multicodec.get("dag-cbor")
+_dag_cbor_code: int = _dag_cbor_multicodec.code
+_dag_cbor_code_bytes: bytes = varint.encode(_dag_cbor_code)
+_dag_cbor_code_nbytes: int = len(_dag_cbor_code_bytes)
+
 @overload
-def encode(data: "EncodableType", stream: None = None) -> bytes:
+def encode(data: "EncodableType", stream: None = None, *, include_multicodec: bool = False) -> bytes:
     ... # pragma: no cover
 
 @overload
-def encode(data: "EncodableType", stream: BufferedIOBase) -> int:
+def encode(data: "EncodableType", stream: BufferedIOBase, *, include_multicodec: bool = False) -> int:
     ... # pragma: no cover
 
-def encode(data: "EncodableType", stream: Optional[BufferedIOBase] = None) -> Union[bytes, int]:
+def encode(data: "EncodableType", stream: Optional[BufferedIOBase] = None, *, include_multicodec: bool = False) -> Union[bytes, int]:
     """
         Encodes the given `data` with the DAG-CBOR codec.
 
@@ -68,12 +83,27 @@ def encode(data: "EncodableType", stream: Optional[BufferedIOBase] = None) -> Un
             def encode(data: EncodableType, stream: None = None) -> bytes:
                 ...
         ```
+
+        If the optional keyword argument `include_multicodec` is `True`, the encoded data includes the multicodec code for 'dag-cbor'
+        (see [`multicodec.wrap`](https://github.com/hashberg-io/multiformats#multicodec)).
+
     """
+    validate(data, EncodableType)
+    validate(stream, Optional[BufferedIOBase])
+    validate(include_multicodec, bool)
     if stream is None:
         internal_stream = BytesIO()
+        if include_multicodec:
+            internal_stream.write(_dag_cbor_code_bytes)
         _encode(internal_stream, data)
         return internal_stream.getvalue()
-    return _encode(stream, data)
+    num_bytes = 0
+    if include_multicodec:
+        stream.write(_dag_cbor_code_bytes)
+        num_bytes += _dag_cbor_code_nbytes
+    num_bytes += _encode(stream, data)
+    return num_bytes
+
 
 def _encode(stream: BufferedIOBase, value: EncodableType) -> int:
     # pylint: disable = too-many-return-statements, too-many-branches
@@ -95,7 +125,7 @@ def _encode(stream: BufferedIOBase, value: EncodableType) -> int:
     if isinstance(value, dict):
         # major type 0x5
         return _encode_dict(stream, value)
-    if isinstance(value, cid.cid.BaseCID):
+    if isinstance(value, CID):
         # major type 0x6
         return _encode_cid(stream, value)
     if value is None:
@@ -152,13 +182,13 @@ def _encode_str(stream: BufferedIOBase, value: str) -> int:
     stream.write(utf8_value)
     return num_head_bytes+len(utf8_value)
 
-def _encode_list(stream: BufferedIOBase, value: list) -> int:
+def _encode_list(stream: BufferedIOBase, value: List[Any]) -> int:
     num_bytes_written = _encode_head(stream, 0x4, len(value))
     for item in value:
         num_bytes_written += _encode(stream, item)
     return num_bytes_written
 
-def _encode_dict(stream: BufferedIOBase, value: dict) -> int:
+def _encode_dict(stream: BufferedIOBase, value: Dict[str, Any]) -> int:
     _check_key_compliance(value)
     # sort keys canonically
     try:
@@ -176,9 +206,9 @@ def _encode_dict(stream: BufferedIOBase, value: dict) -> int:
         num_bytes_written += _encode(stream, v)
     return num_bytes_written
 
-def _encode_cid(stream: BufferedIOBase, value: cid.cid.BaseCID) -> int:
+def _encode_cid(stream: BufferedIOBase, value: CID) -> int:
     num_bytes_written = _encode_head(stream, 0x6, 42)
-    num_bytes_written += _encode_bytes(stream, value.buffer)
+    num_bytes_written += _encode_bytes(stream, bytes(value))
     return num_bytes_written
 
 def _encode_bool(stream: BufferedIOBase, value: bool) -> int:
