@@ -11,7 +11,7 @@
     - `bool_none` for `False`, `True` or `None` (33.3% each)
     - `list` for lists of uniformly distributed length, with random elements of any type
     - `dict` for dictionaries of uniformly distributed length, with distinct random string keys and random values of any type
-    - `cid` for CID data (instance of `BaseCID` from the [`py-cid`](https://github.com/ipld/py-cid) package)
+    - `cid` for CID data (instance of `CID` from the [`multiformats`](https://github.com/hashberg-io/multiformats) library)
 
     The function call `rand_X(n)` returns an iterator yielding a stream of `n` random values of type `X`, e.g.:
 
@@ -76,17 +76,16 @@
 
 
 from contextlib import contextmanager
-from hashlib import sha3_512
 import math
 from random import Random # pylint: disable = import-self
 import sys
 from types import MappingProxyType
-from typing import Any, Dict, Iterator, List, Optional
+from typing import Any, Dict, Iterator, List, Mapping, Optional, Tuple
+from typing_validation import validate
 
-import cid # type: ignore
-import multihash # type: ignore
+from multiformats import multicodec, multibase, multihash, CID
 
-from .encoding import EncodableType
+from .encoding import encode, EncodableType
 from .utils import _canonical_order_dict
 
 _min_int = -18446744073709551616
@@ -127,13 +126,13 @@ def reset_options() -> None:
     _options = _default_options
     _rand = Random(0)
 
-def default_options() -> MappingProxyType:
+def default_options() -> Mapping[str, Any]:
     """
         Readonly view of the default random generation options.
     """
     return MappingProxyType(_default_options)
 
-def get_options() -> MappingProxyType:
+def get_options() -> Mapping[str, Any]:
     """
         Readonly view of the current random generation options.
     """
@@ -157,7 +156,7 @@ def options(*,
             min_float: Optional[float] = None,
             max_float: Optional[float] = None,
             float_decimals: Optional[int] = None,
-            include_cid: Optional[bool] = None,):
+            include_cid: Optional[bool] = None,) -> Iterator[None]:
     """
         Returns with-statement context manager for temporary option setting:
 
@@ -232,6 +231,13 @@ def set_options(*,
 
     """
     # pylint: disable = too-many-branches, too-many-locals, too-many-statements
+    for iarg in (seed, min_int, max_int, min_bytes, max_bytes, min_chars, max_chars,
+                 min_codepoint, max_codepoint, min_len, max_len, max_nesting, float_decimals):
+        validate(iarg, Optional[int])
+    for barg in (canonical, include_cid):
+        validate(barg, Optional[bool])
+    for farg in (min_float, max_float):
+        validate(farg, Optional[float])
     global _options
     global _rand
     # set newly passed options
@@ -336,6 +342,11 @@ def rand_data(n: Optional[int] = None, *, max_nesting: Optional[int] = None) ->
         - integer values >= 0 mean that containers will be generated, with items generated by `random_data(max_nesting=max_nesting-1)`
         - no other values are valid
     """
+    validate(n, Optional[int])
+    validate(max_nesting, Optional[int])
+    return _rand_data(n, max_nesting=max_nesting)
+
+def _rand_data(n: Optional[int] = None, *, max_nesting: Optional[int] = None) -> Iterator[EncodableType]:
     if n is not None and n < 0:
         raise ValueError()
     if max_nesting is None:
@@ -343,15 +354,15 @@ def rand_data(n: Optional[int] = None, *, max_nesting: Optional[int] = None) ->
     elif max_nesting < -1:
         raise ValueError("Value for max_nesting must be >= -1 (with -1 indicating no containers).")
     include_cid = _options["include_cid"]
-    data_generators: List[Iterator] = [
-        rand_list(max_nesting=max_nesting) if max_nesting >= 0 else iter([]),
-        rand_dict(max_nesting=max_nesting) if max_nesting >= 0 else iter([]),
-        rand_int(),
-        rand_bytes(),
-        rand_str(),
-        rand_bool_none(),
-        rand_float(),
-        rand_cid()
+    data_generators: List[Iterator[Any]] = [
+        _rand_list(max_nesting=max_nesting) if max_nesting >= 0 else iter([]),
+        _rand_dict(max_nesting=max_nesting) if max_nesting >= 0 else iter([]),
+        _rand_int(),
+        _rand_bytes(),
+        _rand_str(),
+        _rand_bool_none(),
+        _rand_float(),
+        _rand_cid()
     ]
     num_data_generators = len(data_generators) if include_cid else len(data_generators)-1
     i = 0
@@ -368,7 +379,7 @@ def rand_data(n: Optional[int] = None, *, max_nesting: Optional[int] = None) ->
             raise RuntimeError("All random streams are infinite, this should not happen.") from e
         i += 1
 
-def rand_list(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[list]:
+def rand_list(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[List[Any]]:
     """
         Generates a stream of random `list` data.
         If a number `n` is given, that number of samples is yelded.
@@ -383,6 +394,12 @@ def rand_list(n: Optional[int] = None, *, length: Optional[int] = None, max_nest
         - integer values > 0 mean that containers will be generated as items, with maximum nesting level `max_nesting-1`
         - no other values are valid
     """
+    validate(n, Optional[int])
+    validate(length, Optional[int])
+    validate(max_nesting, Optional[int])
+    return _rand_list(n, length=length, max_nesting=max_nesting)
+
+def _rand_list(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[List[Any]]:
     if n is not None and n < 0:
         raise ValueError()
     if length is not None and length < 0:
@@ -396,10 +413,10 @@ def rand_list(n: Optional[int] = None, *, length: Optional[int] = None, max_nest
     i = 0
     while n is None or i < n:
         _length = length if length is not None else _rand.randint(min_len, max_len)
-        yield list(rand_data(_length, max_nesting=max_nesting-1))
+        yield list(_rand_data(_length, max_nesting=max_nesting-1))
         i += 1
 
-def rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[dict]:
+def rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[Dict[str, Any]]:
     """
         Generates a stream of random `dict` data.
         If a number `n` is given, that number of samples is yelded.
@@ -414,6 +431,12 @@ def rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nest
         - integer values > 0 mean that containers will be generated as values, with maximum nesting level `max_nesting-1`
         - no other values are valid
     """
+    validate(n, Optional[int])
+    validate(length, Optional[int])
+    validate(max_nesting, Optional[int])
+    return _rand_dict(n, length=length, max_nesting=max_nesting)
+
+def _rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nesting: Optional[int] = None) -> Iterator[Dict[str, Any]]:
     # pylint: disable = too-many-locals, too-many-branches
     if n is not None and n < 0:
         raise ValueError()
@@ -447,7 +470,7 @@ def rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nest
         else:
             keys = []
             keys_set = set()
-            str_generator = rand_str()
+            str_generator = _rand_str()
             while len(keys) < _length:
                 try:
                     s = next(str_generator)
@@ -457,7 +480,7 @@ def rand_dict(n: Optional[int] = None, *, length: Optional[int] = None, max_nest
                     keys.append(s)
                     keys_set.add(s)
         # generate dictionary
-        raw_dict = dict(zip(keys, rand_data(_length, max_nesting=max_nesting-1)))
+        raw_dict = dict(zip(keys, _rand_data(_length, max_nesting=max_nesting-1)))
         if canonical:
             yield _canonical_order_dict(raw_dict)
         else:
@@ -469,6 +492,10 @@ def rand_int(n: Optional[int] = None) -> Iterator[int]:
         Generates a stream of random `int` data.
         If a number `n` is given, that number of samples is yelded.
     """
+    validate(n, Optional[int])
+    return _rand_int(n)
+
+def _rand_int(n: Optional[int] = None) -> Iterator[int]:
     if n is not None and n < 0:
         raise ValueError()
     min_int = _options["min_int"]
@@ -485,6 +512,11 @@ def rand_bytes(n: Optional[int] = None, *, length: Optional[int] = None) -> Iter
 
         The optional `length` keyword argument can be used to fix the number of bytes generated.
     """
+    validate(n, Optional[int])
+    validate(length, Optional[int])
+    return _rand_bytes(n, length=length)
+
+def _rand_bytes(n: Optional[int] = None, *, length: Optional[int] = None) -> Iterator[bytes]:
     if n is not None and n < 0:
         raise ValueError()
     if length is not None and length < 0:
@@ -504,6 +536,11 @@ def rand_str(n: Optional[int] = None, *, length: Optional[int] = None) -> Iterat
 
         The optional `length` keyword argument can be used to fix the number of characters generated.
     """
+    validate(n, Optional[int])
+    validate(length, Optional[int])
+    return _rand_str(n, length=length)
+
+def _rand_str(n: Optional[int] = None, *, length: Optional[int] = None) -> Iterator[str]:
     if n is not None and n < 0:
         raise ValueError()
     if length is not None and length < 0:
@@ -529,6 +566,10 @@ def rand_bool(n: Optional[int] = None) -> Iterator[bool]:
         Generates a stream of random `bool` data.
         If a number `n` is given, that number of samples is yelded.
     """
+    validate(n, Optional[int])
+    return _rand_bool(n)
+
+def _rand_bool(n: Optional[int] = None) -> Iterator[bool]:
     if n is not None and n < 0:
         raise ValueError()
     i = 0
@@ -542,6 +583,10 @@ def rand_bool_none(n: Optional[int] = None) -> Iterator[Optional[bool]]:
         Generates a stream of random `Optional[bool]` data.
         If a number `n` is given, that number of samples is yelded.
     """
+    validate(n, Optional[int])
+    return _rand_bool_none(n)
+
+def _rand_bool_none(n: Optional[int] = None) -> Iterator[Optional[bool]]:
     if n is not None and n < 0:
         raise ValueError()
     i = 0
@@ -555,6 +600,10 @@ def rand_float(n: Optional[int] = None) -> Iterator[float]:
         Generates a stream of random `float` data.
         If a number `n` is given, that number of samples is yelded.
     """
+    validate(n, Optional[int])
+    return _rand_float(n)
+
+def _rand_float(n: Optional[int] = None) -> Iterator[float]:
     if n is not None and n < 0:
         raise ValueError()
     min_float = _options["min_float"]
@@ -581,33 +630,56 @@ def rand_float(n: Optional[int] = None) -> Iterator[float]:
             yield x-x%eps
             i += 1
 
-def rand_cid(n: Optional[int] = None) -> Iterator[None]:
-    """
-        Generates a stream of random `CID` data:
+_cid_multibase = multibase.get("base58btc") # the default base for binary CIDs
+_cid_version = 1
+_cid_multicodec = multicodec.get("dag-cbor")
+_cid_multihash = multihash.get("sha3-512")
 
-        - CID v1
-        - codec 'dag-cbor'
-        - hash function 'sha3-512'
-        - raw digest: 512 random bits
+def rand_data_cid(n: Optional[int] = None, *, max_nesting: Optional[int] = None) -> Iterator[Tuple[EncodableType, CID]]:
+    """
+        Generates a stream of random dag-cbor data and associated CIDs:
+
+        - multibase 'base32'
+        - CIDv1
+        - multicodec 'dag-cbor'
+        - multihash 'sha3-512', with full 512-bit digest
 
         If a number `n` is given, that number of samples is yelded.
     """
+    validate(n, Optional[int])
+    return _rand_data_cid(n, max_nesting=max_nesting)
+
+def rand_cid(n: Optional[int] = None, *, max_nesting: Optional[int] = None) -> Iterator[CID]:
+    """
+        Generates a stream of random CIDs:
+
+        - multibase 'base32'
+        - CIDv1
+        - multicodec 'dag-cbor'
+        - multihash 'sha3-512', with full 512-bit digest
+
+        If a number `n` is given, that number of samples is yelded.
+    """
+    validate(n, Optional[int])
+    return _rand_cid(n, max_nesting=max_nesting)
+
+def _rand_cid(n: Optional[int] = None, *, max_nesting: Optional[int] = None) -> Iterator[CID]:
+    return (cid for _, cid in _rand_data_cid(n, max_nesting=max_nesting))
+
+def _rand_data_cid(n: Optional[int] = None, *, max_nesting: Optional[int] = None) -> Iterator[Tuple[EncodableType, CID]]:
     if n is not None and n < 0:
         raise ValueError()
-    hashfun = sha3_512
-    hashcode = "sha3-512"
-    hashlen = 0x40
-    cid_version = 1
-    cid_codec = "dag-cbor"
-    bytes_generator = rand_bytes(length=hashlen)
+    if max_nesting is None:
+        max_nesting = _options["max_nesting"]
+    elif max_nesting < 0:
+        raise ValueError("Value for max_nesting is negative.")
     i = 0
+    rand_data_generator = _rand_data(max_nesting=max_nesting-1)
     while n is None or i < n:
         try:
-            payload = next(bytes_generator)
-            h = hashfun()
-            h.update(payload)
-            digest = h.digest()
+            dag_cbor_data = next(rand_data_generator)
+            binary_data = encode(dag_cbor_data)
         except StopIteration as e:
             raise RuntimeError("Random digest stream is infinite, this should not happen.") from e
-        yield cid.make_cid(cid_version, cid_codec, multihash.encode(digest, hashcode))
+        yield (dag_cbor_data, CID(_cid_multibase, _cid_version, _cid_multicodec, _cid_multihash.digest(binary_data)))
         i += 1
