@@ -10,7 +10,7 @@ from dag_cbor.random import rand_list, rand_dict, rand_int, rand_bytes, rand_str
 
 nsamples = 1000
 
-def test_int():
+def test_int() -> None:
     """
         Encodes random `int` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -20,7 +20,7 @@ def test_int():
         error_msg = f"failed at #{i} = {repr(x)}"
         assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_bytes():
+def test_bytes() -> None:
     """
         Encodes random `bytes` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -30,7 +30,7 @@ def test_bytes():
         error_msg = f"failed at #{i} = {repr(x)}"
         assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_str():
+def test_str() -> None:
     """
         Encodes random `str` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -40,7 +40,7 @@ def test_str():
         error_msg = f"failed at #{i} = {repr(x)}"
         assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_bool_none():
+def test_bool_none() -> None:
     """
         Encodes random `Optional[bool]` or samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -50,7 +50,7 @@ def test_bool_none():
         error_msg = f"failed at #{i} = {repr(x)}"
         assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_float():
+def test_float() -> None:
     """
         Encodes random `float` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -60,7 +60,7 @@ def test_float():
         error_msg = f"failed at #{i} = {repr(x)}"
         assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_list():
+def test_list() -> None:
     """
         Encodes random `list` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -71,7 +71,7 @@ def test_list():
             error_msg = f"failed at #{i} = {repr(x)}"
             assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_dict():
+def test_dict() -> None:
     """
         Encodes random `dict` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -82,7 +82,7 @@ def test_dict():
             error_msg = f"failed at #{i} = {repr(x)}"
             assert cbor2.dumps(x) == encode(x), error_msg
 
-def test_cid():
+def test_cid() -> None:
     """
         Encodes random CID samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -90,4 +90,4 @@ def test_cid():
     test_data = rand_cid(nsamples)
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
-        assert cbor2.dumps(cbor2.CBORTag(42, x.buffer)) == encode(x), error_msg
+        assert cbor2.dumps(cbor2.CBORTag(42, bytes(x))) == encode(x), error_msg
