@@ -8,7 +8,7 @@ from dag_cbor.random import rand_list, rand_dict, rand_int, rand_bytes, rand_str
 
 nsamples = 1000
 
-def test_int():
+def test_int() -> None:
     """
         Encodes random `int` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -17,8 +17,9 @@ def test_int():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_bytes():
+def test_bytes() -> None:
     """
         Encodes random `bytes` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -27,8 +28,9 @@ def test_bytes():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_str():
+def test_str() -> None:
     """
         Encodes random `str` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -37,8 +39,9 @@ def test_str():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_bool_none():
+def test_bool_none() -> None:
     """
         Encodes random `Optional[bool]` or samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -47,8 +50,9 @@ def test_bool_none():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_float():
+def test_float() -> None:
     """
         Encodes random `float` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -57,30 +61,33 @@ def test_float():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_list():
+def test_list() -> None:
     """
         Encodes random `list` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
     """
     with options(include_cid=False):
         test_data = rand_list(nsamples)
-        for i, x in enumerate(test_data):
-            error_msg = f"failed at #{i} = {repr(x)}"
-            assert x == decode(encode(x)), error_msg
+    for i, x in enumerate(test_data):
+        error_msg = f"failed at #{i} = {repr(x)}"
+        assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_dict():
+def test_dict() -> None:
     """
         Encodes random `dict` samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
     """
     with options(include_cid=False):
         test_data = rand_dict(nsamples)
-        for i, x in enumerate(test_data):
-            error_msg = f"failed at #{i} = {repr(x)}"
-            assert x == decode(encode(x)), error_msg
+    for i, x in enumerate(test_data):
+        error_msg = f"failed at #{i} = {repr(x)}"
+        assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
 
-def test_cid():
+def test_cid() -> None:
     """
         Encodes random CID samples with `dag_cbor.encoding.encode`,
         encodes them with `cbor2.encoder.dumps` and checks that the two encodings match.
@@ -89,3 +96,4 @@ def test_cid():
     for i, x in enumerate(test_data):
         error_msg = f"failed at #{i} = {repr(x)}"
         assert x == decode(encode(x)), error_msg
+        assert x == decode(encode(x, include_multicodec=True), require_multicodec=True), error_msg
